(*----------------------------------------------------------------------------*
 *  Direct3D sample for DirectX 8.x demostrating renderer for font format     *
 *     from Nitro: http://www.sulaco.co.za/nitrogen/                          *
 *  by Alexey Barkovoy                                                        *
 *  E-Mail: clootie@reactor.ru                                                *
 *                                                                            *
 *  This example demonstrates use of TD3DNitroFont class which has syntax     *
 *  very close to CD3DFont from DirectX 8 SDK samples framework               *
 *  Sample itself is converted to DirectX usage sample from Nitrogen site -   *
 *  this will help you to see differences between DirectX and OpenGL versions *
 *                                                                            *
 *  Modified: 17-Mar-2002                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru                                                *
 *----------------------------------------------------------------------------*)

{$I DirectX.inc}

{$IFDEF TMT}
unit Demo1Unt;
{$ELSE}
unit Demo1Unit;
{$ENDIF}

interface

uses
  Windows, Messages, CommDlg,{$IFNDEF TMT} SysUtils,{$ENDIF}
  D3DX8, D3DApp, D3DFont, D3DFile, D3DUtil, DXUtil, {$I UseD3D8.inc},
  NitroFnt;

// C++Builder compile hack
{$NOINCLUDE NitroFnt}
{$HPPEMIT '#include "NitroFnt.hpp"'}

//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------

type
  // Custom D3D vertex format used by the vertex buffer
  PMyVertex = ^TMyVertex;
  TMyVertex = packed record
    p: TD3DXVector3;       // vertex position
    n: TD3DXVector3;       // vertex normal
  end;

  PMyVertexArray = ^TMyVertexArray;
  TMyVertexArray = array [0..MaxInt div SizeOf(TMyVertex) - 1] of TMyVertex;

const
  D3DFVF_MYVERTEX = (D3DFVF_XYZ or D3DFVF_NORMAL);

type
//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the
//       generic functionality needed in all Direct3D samples. CMyD3DApplication
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
{$IFDEF TMT}
  PMyD3DApplication = ^CMyD3DApplication;
  CMyD3DApplication = object(TD3DApplication)
{$ELSE}
  CMyD3DApplication = class(CD3DApplication)
{$ENDIF}
{$IFDEF TMT}
    m_pFont: PD3DNitroFont;
{$ELSE}
    m_pFont: TD3DNitroFont;
{$ENDIF}
    m_pStatsFont: CD3DFont;
    m_strFont: array[0..99] of Char;
    m_dwFontSize: DWORD;

    m_matObj1: TD3DXMatrix;
    m_matObj2: TD3DXMatrix;
    m_matObj3: TD3DXMatrix;
    m_matObj4: TD3DXMatrix;
    m_matObj5: TD3DXMatrix;
{$IFDEF TMT}
  public
{$ELSE}
  protected
{$ENDIF}
    function OneTimeSceneInit: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function InitDeviceObjects: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function RestoreDeviceObjects: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function InvalidateDeviceObjects: HRESULT;{$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function DeleteDeviceObjects: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function Render: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function FrameMove: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function FinalCleanup: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};

  public
    constructor Create;
    function ConfirmDevice(var p1: TD3DCaps8; p2: DWORD;
      p3: TD3DFormat): HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
  end;

implementation

constructor CMyD3DApplication.Create;
begin
  inherited Create(0);

  m_strWindowTitle    := 'Text3D: Text in a 3D scene';
  m_bUseDepthBuffer   := False;

  // Create fonts
  m_strFont:= 'Arial';
  m_dwFontSize := 18;

  {$IFDEF TMT}
  New(m_pFont, Create('test.fnt'));
  New(m_pStatsFont, Create('Arial', 12, D3DFONT_BOLD));
  {$ELSE}
  m_pFont      := TD3DNitroFont.Create('test.fnt');
  m_pStatsFont := CD3DFont.Create('Arial', 12, D3DFONT_BOLD);
  {$ENDIF}

end;

//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
begin
  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
begin
  // Setup five rotation matrices (for rotating text strings)
  D3DXMatrixRotationAxis(m_matObj1, D3DXVector3(1.0,2.0,0.0), m_fTime/1.1);
  D3DXMatrixRotationAxis(m_matObj2, D3DXVector3(2.0,1.0,0.0), m_fTime/1.2);
  D3DXMatrixRotationAxis(m_matObj3, D3DXVector3(1.0,0.0,2.0), m_fTime/1.3);
  D3DXMatrixRotationAxis(m_matObj4, D3DXVector3(2.0,0.0,1.0), m_fTime/1.4);
  D3DXMatrixRotationAxis(m_matObj5, D3DXVector3(0.0,1.0,2.0), m_fTime/1.5);

  // Add some translational values to the matrices
  m_matObj1._41 := 1.0;   m_matObj1._42 := 3.0;   m_matObj1._43 := 0.0;
  m_matObj2._41 := 2.0;   m_matObj2._42 :=-2.0;   m_matObj2._43 := 0.0;
  m_matObj3._41 := 3.0;   m_matObj3._42 := 1.0;   m_matObj3._43 := 0.0;
  m_matObj4._41 :=-1.0;   m_matObj4._42 :=-3.0;   m_matObj4._43 := 0.0;
  m_matObj5._41 :=-2.0;   m_matObj5._42 := 2.0;   m_matObj5._43 := 0.0;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  mtrl: TD3DMaterial8;
begin
  // Clear the viewport
//  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, $00000000, 1.0, 0);
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, $00FF00FF, 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Draw the 5 spinning, colored 3D text objects
    D3DUtil_InitMaterial(mtrl, 1.0, 0.0, 0.0, 1.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj1);
    // m_pFont.Render3DText('Red Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont{$IFDEF TMT}^{$ENDIF}.Render3DText('Red Text', [rfCentered, rfTwoSided, rfFiltered]);

    D3DUtil_InitMaterial(mtrl, 0.0, 1.0, 0.0, 1.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj2);
    // m_pFont.Render3DText('Green Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont{$IFDEF TMT}^{$ENDIF}.Render3DText('Green Text', [rfCentered, rfTwoSided, rfFiltered]);

    D3DUtil_InitMaterial(mtrl, 0.0, 0.0, 1.0, 1.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj3);
    // m_pFont.Render3DText('Blue Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont{$IFDEF TMT}^{$ENDIF}.Render3DText('Blue Text', [rfCentered, rfTwoSided, rfFiltered]);

    D3DUtil_InitMaterial(mtrl, 1.0, 0.0, 1.0, 1.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj4);
    // m_pFont.Render3DText('Purple Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont{$IFDEF TMT}^{$ENDIF}.Render3DText('Purple Text', [rfCentered, rfTwoSided, rfFiltered]);

    D3DUtil_InitMaterial(mtrl, 1.0, 1.0, 0.0, 1.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj5);
    // m_pFont.Render3DText('Yellow Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont{$IFDEF TMT}^{$ENDIF}.Render3DText('Yellow Text', [rfCentered, rfTwoSided, rfFiltered]);

    // Draw some 2D text
    m_pFont{$IFDEF TMT}^{$ENDIF}.DrawText(2, 40, $ffffffff, '2D Text', []);

    D3DUtil_InitMaterial(mtrl, 1.0, 0.0, 1.0, 1.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pFont{$IFDEF TMT}^{$ENDIF}.DrawTextScaled(-0.9, 0.7, 1, 0.06, 0.06, $FFFFFFFF,
      'This text is scaled to window height', []);


    // Show frame rate
    m_pStatsFont{$IFDEF TMT}^{$ENDIF}.DrawText(2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats{$IFDEF TMT}, 0{$ENDIF});
    m_pStatsFont{$IFDEF TMT}^{$ENDIF}.DrawText(2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats{$IFDEF TMT}, 0{$ENDIF});

    // End the scene.
    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
begin
  // Restore the fonts
  m_pFont{$IFDEF TMT}^{$ENDIF}.InitDeviceObjects(m_pd3dDevice);
  m_pStatsFont{$IFDEF TMT}^{$ENDIF}.InitDeviceObjects(m_pd3dDevice);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  vEyePt: TD3DXVector3;
  vLookatPt: TD3DXVector3;
  vUpVec: TD3DXVector3;
  matWorld, matView, matProj: TD3DXMatrix;
  fAspect: Single;
begin
  // Restore the fonts
  m_pFont{$IFDEF TMT}^{$ENDIF}.RestoreDeviceObjects;
  m_pStatsFont{$IFDEF TMT}^{$ENDIF}.RestoreDeviceObjects;

  // Restore the textures
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetRenderState(D3DRS_DITHERENABLE, 1);
  m_pd3dDevice.SetRenderState(D3DRS_SPECULARENABLE, 1);
  m_pd3dDevice.SetRenderState(D3DRS_AMBIENT,  $00ffffff);

  // Set the transform matrices
  vEyePt    := D3DXVector3(0.0,-5.0,-10.0);
  vLookatPt := D3DXVector3(0.0, 0.0,  0.0);
  vUpVec    := D3DXVector3(0.0, 1.0,  0.0);

  D3DXMatrixIdentity(matWorld);
  D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
  fAspect := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  D3DXMatrixPerspectiveFovLH(matProj, D3DX_PI/4, fAspect, 1.0, 100.0);

  m_pd3dDevice.SetTransform(D3DTS_WORLD,      matWorld);
  m_pd3dDevice.SetTransform(D3DTS_VIEW,       matView);
  m_pd3dDevice.SetTransform(D3DTS_PROJECTION, matProj);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  m_pFont{$IFDEF TMT}^{$ENDIF}.InvalidateDeviceObjects;
  m_pStatsFont{$IFDEF TMT}^{$ENDIF}.InvalidateDeviceObjects;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
begin
  m_pFont{$IFDEF TMT}^{$ENDIF}.DeleteDeviceObjects;
  m_pStatsFont{$IFDEF TMT}^{$ENDIF}.DeleteDeviceObjects;
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  SAFE_DELETE(m_pFont);
  SAFE_DELETE(m_pStatsFont);

  Result:= S_OK;
end;

function CMyD3DApplication.ConfirmDevice(var p1: TD3DCaps8; p2: DWORD;
  p3: TD3DFormat): HRESULT;
begin
  Result:= S_OK;
  if (p2 and D3DCREATE_PUREDEVICE)<> 0 then Result:= E_FAIL;
end;

end.