(*----------------------------------------------------------------------------*
 *  Direct3D sample for DirectX 8.x demostrating renderer for font format     *
 *     from Nitro: http://www.sulaco.co.za/nitrogen/                          *
 *  by Alexey Barkovoy                                                        *
 *  E-Mail: clootie@reactor.ru                                                *
 *                                                                            *
 *  This example demonstrates use of TD3DNitroFont class which has syntax     *
 *  very close to CD3DFont from DirectX 8 SDK samples framework               *
 *  Sample itself is converted to DirectX usage sample from Nitrogen site -   *
 *  this will help you to see differences between DirectX and OpenGL versions *
 *                                                                            *
 *  Modified: 17-Mar-2002                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru                                                *
 *----------------------------------------------------------------------------*)

{$I DirectX.inc}

unit Demo2Unit;

interface

uses
  Windows, Messages, CommDlg,{$IFNDEF TMT} SysUtils,{$ENDIF}
  D3DX8, D3DApp, D3DFont, D3DFile, D3DUtil, DXUtil, NitroFnt,
  {$I UseD3D8.inc};

// C++Builder compile hack
{$NOINCLUDE NitroFnt}
{$HPPEMIT '#include "NitroFnt.hpp"'}

//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------

type
  // Custom D3D vertex format used by the vertex buffer
  PMyVertex = ^TMyVertex;
  TMyVertex = packed record
    p: TD3DXVector3;       // vertex position
    n: TD3DXVector3;       // vertex normal
  end;

  PMyVertexArray = ^TMyVertexArray;
  TMyVertexArray = array [0..MaxInt div SizeOf(TMyVertex) - 1] of TMyVertex;

const
  D3DFVF_MYVERTEX = (D3DFVF_XYZ or D3DFVF_NORMAL);

type
//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the
//       generic functionality needed in all Direct3D samples. CMyD3DApplication
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
{$IFDEF TMT}
  PMyD3DApplication = ^CMyD3DApplication;
  CMyD3DApplication = object(TD3DApplication)
    m_pFont: PD3DNitroFont;
{$ELSE}
  CMyD3DApplication = class(CD3DApplication)
    m_pFont: TD3DNitroFont;
{$ENDIF}
    m_pStatsFont: CD3DFont;

    m_matObj1: TD3DXMatrix;
    m_matObj2: TD3DXMatrix;

{$IFDEF TMT}
  public
{$ELSE}
  protected
{$ENDIF}
    function OneTimeSceneInit: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function InitDeviceObjects: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function RestoreDeviceObjects: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function InvalidateDeviceObjects: HRESULT;{$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function DeleteDeviceObjects: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function Render: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function FrameMove: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};
    function FinalCleanup: HRESULT; {$IFDEF TMT}virtual{$ELSE}override{$ENDIF};

  public
    // LRESULT MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
    constructor Create;
    procedure ViewOrtho;
  end;

implementation

constructor CMyD3DApplication.Create;
begin
  inherited Create(0);

  m_strWindowTitle    := 'Nitro font format DirectX renderer';
  m_bUseDepthBuffer   := True;

  m_dwCreationWidth:= 640;
  m_dwCreationHeight:= 480;

  // Create fonts
  {$IFDEF TMT}
  New(m_pFont, Create('test.fnt'));
  New(m_pStatsFont, Create('Arial', 12, D3DFONT_BOLD));
  {$ELSE}
  m_pFont      := TD3DNitroFont.Create('test.fnt');
  m_pStatsFont := CD3DFont.Create('Arial', 12, D3DFONT_BOLD);
  {$ENDIF}
end;

//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
begin
  Result:= S_OK;
end;

procedure CMyD3DApplication.ViewOrtho;
var
  mat: TD3DXMatrix;
  R: TRect;
begin
  R:= m_rcWindowClient;

  // Select Ortho Mode
  D3DXMatrixOrthoOffCenterLH(mat, 0, R.Right, 0, R.Bottom, 0.1, 1000);

  m_pd3dDevice.SetTransform(D3DTS_PROJECTION, mat);
  D3DXMatrixIdentity(mat);
  m_pd3dDevice.SetTransform(D3DTS_VIEW, mat);
  m_pd3dDevice.SetTransform(D3DTS_WORLD, mat);
end;

//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
begin
  // Setup five rotation matrices (for rotating text strings)
  D3DXMatrixRotationAxis(m_matObj1, D3DXVector3(1.0,2.0,0.0), m_fTime/1.1);
  D3DXMatrixRotationAxis(m_matObj2, D3DXVector3(2.0,1.0,0.0), m_fTime/1.2);

  // Add some translational values to the matrices
  m_matObj1._41 := 1.0;   m_matObj1._42 := 3.0;   m_matObj1._43 := 0.0;
  m_matObj2._41 := 2.0;   m_matObj2._42 :=-2.0;   m_matObj2._43 := 0.0;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
type
  TCustomVertex = packed record
    p: TD3DXVector3;
    color: DWORD;
  end;
const
  D3DFVF_Quad = (D3DFVF_XYZ or D3DFVF_DIFFUSE);
var
  m0, m1: TD3DXMatrix;
  mtrl: TD3DMaterial8;
  Quad: array[0..3] of TCustomVertex;
  ScrWidth, ScrHeight: Integer;
  ElapsedTime: Single;
  txt: String;
  txtLength: Single;
  i: Integer;
begin
  ScrWidth:= m_rcWindowClient.Right;
  ScrHeight:= m_rcWindowClient.Bottom;
  ElapsedTime:= m_fTime * 1000;


  // Clear the viewport
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, $00000000, 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iFalse);

    D3DXMatrixTranslation(m0, 0, 0, 1);
    m_pd3dDevice.SetTransform(D3DTS_VIEW, m0);
    D3DXMatrixIdentity(m0);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m0);

    m_pd3dDevice.SetVertexShader(D3DFVF_Quad);

    Quad[0].p:= D3DXVector3(ScrWidth/2 - 250, ScrHeight/2 + 130, 5);
    Quad[1].p:= D3DXVector3(ScrWidth/2 + 250, ScrHeight/2 + 130, 5);
    Quad[2].p:= D3DXVector3(ScrWidth/2 + 250, ScrHeight/2 - 130, 5);
    Quad[3].p:= D3DXVector3(ScrWidth/2 - 250, ScrHeight/2 - 130, 5);
    Quad[0].color:= D3DCOLOR_COLORVALUE(0.2, 0.5, 0.4, 1);
    Quad[1].color:= D3DCOLOR_COLORVALUE(0.2, 0.5, 0.4, 1);
    Quad[2].color:= D3DCOLOR_COLORVALUE(0.0, 0.0, 0.0, 1);
    Quad[3].color:= D3DCOLOR_COLORVALUE(0.0, 0.0, 0.0, 1);
{$IFNDEF DXG_COMPAT}
    m_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, Quad, SizeOf(TCustomVertex));
{$ELSE}
    m_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, @Quad, SizeOf(TCustomVertex));
{$ENDIF}

    Quad[0].p:= D3DXVector3( 40, ScrHeight-50, 5);
    Quad[1].p:= D3DXVector3(600, ScrHeight-50, 5);
    Quad[2].p:= D3DXVector3(600, ScrHeight-95, 5);
    Quad[3].p:= D3DXVector3( 40, ScrHeight-95, 5);
    Quad[0].color:= D3DCOLOR_COLORVALUE(0.9, 0.5, 0.4, 1);
    Quad[1].color:= D3DCOLOR_COLORVALUE(0.2, 0.2, 0.9, 1);
    Quad[2].color:= D3DCOLOR_COLORVALUE(0.2, 0.2, 0.9, 1);
    Quad[3].color:= D3DCOLOR_COLORVALUE(0.9, 0.5, 0.4, 1);
{$IFNDEF DXG_COMPAT}
    m_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, Quad, SizeOf(TCustomVertex));
{$ELSE}
    m_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, @Quad, SizeOf(TCustomVertex));
{$ENDIF}

    m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iTrue);
    D3DUtil_InitMaterial(mtrl, 1.0, 1.0, 1.0{$IFDEF TMT}, 1.0{$ENDIF});
    m_pd3dDevice.SetMaterial(mtrl);

    m_pFont{$IFDEF TMT}^{$ENDIF}.Draw(50, ScrHeight-50, 'Visit http://www.sulaco.co.za/nitrogen/', -1{$IFDEF TMT}, [], False{$ENDIF});
    m_pFont{$IFDEF TMT}^{$ENDIF}.Draw(50, ScrHeight-70, 'for OpenGL renderer and FontStudio', -1{$IFDEF TMT}, [], False{$ENDIF});

    D3DXMatrixMultiply(m0,
      D3DXMatrixRotationAxis(m0,
        D3DXVector3(0,0,1),
        D3DXToRadian(Sin(ElapsedTime/300)*20 + Cos(ElapsedTime/500)*40))^,
      D3DXMatrixTranslation(m1,
        ScrWidth/2+Sin(ElapsedTime/300)*50, ScrHeight/2, 0)^
      );
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m0);

    txt:= 'Visit http://clootie.narod.ru/';
    txtLength:= m_pFont{$IFDEF TMT}^{$ENDIF}.TextLen(Txt) / 2;

    m_pFont{$IFDEF TMT}^{$ENDIF}.Draw(-txtLength, 10, Txt, -2{$IFDEF TMT}, [], False{$ENDIF});


    m_pFont{$IFDEF TMT}^{$ENDIF}.PrepareDrawing([]);
    m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,         D3DBLEND_ONE);
    m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND,        D3DBLEND_ONE);
    m_pd3dDevice.SetRenderState(D3DRS_ALPHATESTENABLE,  iTrue);
    m_pd3dDevice.SetRenderState(D3DRS_ALPHAREF,         $01);

    for i := 0 to 20 do
    begin
      D3DUtil_InitMaterial(mtrl, i/200, i/200, i/100{$IFDEF TMT}, 1.0{$ENDIF});
      m_pd3dDevice.SetMaterial(mtrl);

      D3DXMatrixMultiply(m0,
        D3DXMatrixRotationAxis(m0,
          D3DXVector3(0,0,1),
          D3DXToRadian(Sin(ElapsedTime/300-i/20)*20 + Cos(ElapsedTime/500-i/20)*40))^,
        D3DXMatrixTranslation(m1,
          ScrWidth/2+Sin(ElapsedTime/300-i/20)*50, ScrHeight/2, 0)^
        );
      m_pd3dDevice.SetTransform(D3DTS_WORLD, m0);

      m_pFont{$IFDEF TMT}^{$ENDIF}.Draw(-txtLength, 10, Txt, 0, [], True);
    end;
    m_pFont{$IFDEF TMT}^{$ENDIF}.RestoreState;

    // Show frame rate
    m_pStatsFont{$IFDEF TMT}^{$ENDIF}.DrawText(2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats{$IFDEF TMT}, 0{$ENDIF});
    m_pStatsFont{$IFDEF TMT}^{$ENDIF}.DrawText(2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats{$IFDEF TMT}, 0{$ENDIF});

    // End the scene.
    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
begin
  // Restore the fonts
  m_pFont{$IFDEF TMT}^{$ENDIF}.InitDeviceObjects(m_pd3dDevice);
  m_pStatsFont{$IFDEF TMT}^{$ENDIF}.InitDeviceObjects(m_pd3dDevice);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
begin
  // Restore the fonts
  m_pFont{$IFDEF TMT}^{$ENDIF}.RestoreDeviceObjects;
  m_pStatsFont{$IFDEF TMT}^{$ENDIF}.RestoreDeviceObjects;

  // Restore the textures
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_current);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_SELECTARG2);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_current);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG2);

  m_pd3dDevice.SetTextureStageState(1, D3DTSS_COLOROP,   D3DTOP_DISABLE);
  m_pd3dDevice.SetTextureStageState(1, D3DTSS_ALPHAOP,   D3DTOP_DISABLE);

  m_pd3dDevice.SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetRenderState(D3DRS_DITHERENABLE, 1);
  m_pd3dDevice.SetRenderState(D3DRS_SPECULARENABLE, 1);
  m_pd3dDevice.SetRenderState(D3DRS_AMBIENT,  $00ffffff);

  // Set the transform matrices
  ViewOrtho;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  m_pFont{$IFDEF TMT}^{$ENDIF}.InvalidateDeviceObjects;
  m_pStatsFont{$IFDEF TMT}^{$ENDIF}.InvalidateDeviceObjects;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
begin
  m_pFont{$IFDEF TMT}^{$ENDIF}.DeleteDeviceObjects;
  m_pStatsFont{$IFDEF TMT}^{$ENDIF}.DeleteDeviceObjects;
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  SAFE_DELETE(m_pFont);
  SAFE_DELETE(m_pStatsFont);

  Result:= S_OK;
end;

end.
